# Copyright (c) 2022 European Union.
#
# The tool was developed with the contribution of the Joint Research Centre of the European Commission.
#
# This program is free software: you can redistribute it and/or modify it under the terms of the European Union Public
# Licence, either version 1.2 of the License, or (at your option) any later version.
# You may not use this work except in compliance with the Licence.
#
# You may obtain a copy of the Licence at: https://joinup.ec.europa.eu/collection/eupl/eupl-guidelines-faq-infographics
#
# Unless required by applicable law or agreed to in writing, software distributed under the Licence is distributed on
# an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the Licence for the specific language governing permissions and limitations under the Licence.

#Doing the actual calc of SDM model
#Get list of  lambda files
import glob
import subprocess
import pandas as pd
import numpy as np

year = 2018
print(year)

swd_file = f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EOBS/AOI_swd_{str(year)}.csv'
lambda_list = glob.glob("/data/inca_vol1/input_data/crop_pollination/input_intermediate/SDM_LAMBDAS/*.lambdas")
outputfiles = []
species_list= []
for lambdas in lambda_list:
    print(f"processing {lambdas}")
    outputname = f"/data/inca_vol1/input_data/crop_pollination/input_intermediate/SDM/not_aggregated/{str(year)}/{str(year)}_{lambdas.split('/')[-1].split('.')[0]}.asc"
    species_list.append(lambdas.split('/')[-1].split('.')[0].rsplit('_',1)[0])
    cmd  = f'java -mx1024m -cp /home/deroob/maxent/maxent.jar density.Project "{lambdas}" "{swd_file}" {outputname} outputformat=Cloglog maximumbackground=10000 replicates=5 replicatetype=crossvalidate "applyThresholdRule=Minimum training presence"'

    try:
        print('')
        subprocess.check_call(cmd, shell=True)
    except Exception:
        raise OSError('Could not run model')

MinimumTrainingThreshold =pd.read_csv('/data/inca_vol1/input_data/crop_pollination/input_raw/SDM/MinimumTrainingThresholds.csv', index_col = 0)
#print(MinimumTrainingThreshold)
#converting to aggregated file 1 per species
total_probability = []
total_presences = []
for specie in set(species_list):
    species_files = glob.glob(f"/data/inca_vol1/input_data/crop_pollination/input_intermediate/SDM/not_aggregated/{str(year)}/{str(year)}_{specie}_?.asc")
    specie_thres = MinimumTrainingThreshold.loc[(f"{specie} (average)"), :].values

    nlist = []
    for model in species_files :
        with open(model) as data:
            adata = (data.readlines())[6].split(' ')
            nlist.append(adata)
            #len(nlist[0,:])

    narray = np.array(nlist,dtype=np.float64)

    narray = np.ma.masked_where(narray==-9999,narray)
    aggre = narray.mean(axis=0)
    total_probability.append(aggre)
    presencemap = np.zeros_like(aggre)
    presencemap[aggre >= specie_thres] = 1
    total_presences.append(presencemap)
    #some intermed output
    df = pd.DataFrame(aggre)
    df.to_csv(f"/data/inca_vol1/input_data/crop_pollination/input_intermediate/SDM/aggregated/{str(year)}/{str(year)}_{specie}.csv")
    df2 = pd.DataFrame(presencemap)
    df2.to_csv(f"/data/inca_vol1/input_data/crop_pollination/input_intermediate/SDM/presencemaps/{str(year)}/{str(year)}_{specie}.csv")

print(np.shape(total_presences))
total_presences = np.array(total_presences)
total_probability = np.array(total_probability)
species_richness = np.sum(total_presences,axis=0)
pd.DataFrame(species_richness).to_csv(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/SDM/{year}_species_richness.csv')
total_probability[total_presences==0] = 0
print(np.shape(total_probability))
aggregated_probability = np.sum(total_probability, axis = 0)
print(np.shape(aggregated_probability))
print(np.shape(species_richness))
Avg_prob_occ = aggregated_probability /species_richness
print(np.shape(Avg_prob_occ))
Avg_prob_occ[species_richness==0] = 0
print(np.shape(Avg_prob_occ))
pd_apo = pd.DataFrame(Avg_prob_occ)
pd_apo.to_csv(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/SDM/{year}_average_probability_occurence.csv')

import sys
sys.exit(0)
