#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2022 European Union.
#
# The tool was developed with the contribution of the Joint Research Centre of the European Commission.
#
# This program is free software: you can redistribute it and/or modify it under the terms of the European Union Public
# Licence, either version 1.2 of the License, or (at your option) any later version.
# You may not use this work except in compliance with the Licence.
#
# You may obtain a copy of the Licence at: https://joinup.ec.europa.eu/collection/eupl/eupl-guidelines-faq-infographics
#
# Unless required by applicable law or agreed to in writing, software distributed under the Licence is distributed on
# an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the Licence for the specific language governing permissions and limitations under the Licence.


"""
SYNOPSIS
    EOBS_preprocessing.py [-v,--verbose] [-h,--help] [--version]

DESCRIPTION
    This snippet is for projecting the SDM model with existing lambda files to later time steps
    in preparation of the crop pollination model run.

PREREQUISITES
    Python => 3.7.0

EXAMPLES
    python -m inca.croppollination.preprocessing.EOBS_preprocessing
    OR
    python3 EOBS_preprocessing.py

EXIT STATUS
    Ctrl-C  =  cancel the script at any time

AUTHOR
    Dr. Bert De Roo <bert.deroo@vito.be>

LICENSE
    This script is property of VITO. Copyrights or restrictions may apply.

VERSION
    0.1 (2021-11-18)
"""
import numpy as np

''' STILL a dummy '''
import rasterio
from datetime import date
import pandas as pd
from affine import Affine
year = 2012
#bio4 standard deviation of temp *100

f_date = date(1950, 1, 1)
l_date = date(year-21, 1, 1)



def calc_month_func(rasterstack, func =np.ma.average):
    days = np.shape(rasterstack)[0]
    dtype = rasterstack.dtype
    if days == 365:
        feb = 28
    elif days ==366:
        feb = 29
    else: raise(f"something went terrible wrong. days is {days }")


    start = 0
    monthly_func = np.ma.zeros((12,np.shape(rasterstack)[1],np.shape(rasterstack)[2]),dtype=dtype)
    for idx,month in enumerate([31,feb,31,30,31,30,31,31,30,31,30,31]):
        monthly_func[idx] = func(rasterstack[start:start+month,:,:],axis=0)
        start = start +month
    return monthly_func





#bio4 standard deviation of temp *100
with rasterio.open('/data/inca_vol1/input_data/crop_pollination/input_raw/EOBS/tx_ens_mean_0.25deg_reg_v24.0e.nc') as maxt_src, \
    rasterio.open('/data/inca_vol1/input_data/crop_pollination/input_raw/EOBS/tn_ens_mean_0.25deg_reg_v24.0e.nc') as mint_src, \
    rasterio.open('/data/inca_vol1/input_data/crop_pollination/input_raw/EOBS/rr_ens_mean_0.25deg_reg_v24.0e.nc') as perc_src:
    dst_profile = maxt_src.profile

    maxt_scale = maxt_src.scales[0]
    mint_scale = mint_src.scales[0]
    perc_scale = perc_src.scales[0]

    maxt_yearly = np.ma.zeros((22,12, maxt_src.height,maxt_src.width), dtype=np.float64)
    mint_yearly = np.zeros_like(maxt_yearly)
    perc_yearly = np.zeros_like(maxt_yearly)
    for idx, getyear in enumerate(range(l_date.year, year+1)):
        print(f'getting {getyear} data' )
        beginband = (date(getyear, 1, 1) - f_date).days + 1
        endband = (date(getyear, 12, 31) - f_date).days + 2
        bandlist = range(beginband, endband)
        maxt_yearly[idx] = calc_month_func(maxt_src.read(bandlist,masked=True), )*maxt_scale
        mint_yearly[idx] = calc_month_func(mint_src.read(bandlist,masked=True))*mint_scale
        perc_yearly[idx] = calc_month_func(perc_src.read(bandlist,masked=True),func= np.ma.sum)*perc_scale

maxt = np.ma.mean(maxt_yearly,axis=0)
mint = np.ma.mean(mint_yearly,axis=0)
perc = np.ma.mean(perc_yearly,axis=0)


#calc monthly aveage temp
avgt = (maxt + mint) / 2.
bio4 = (np.ma.std(avgt,axis = 0)*100).filled(-9999)
dst_profile.update({'count': 14, 'dtype': np.float64})

dst_profile.update({'count': 1, 'driver': 'GTiff', 'dtype': np.float64, 'nodata': -9999, 'crs': 'EPSG:4326'})
'''
Could not find a pixel shift
new_affine = dst_profile.pop('transform')

dst_profile.update(transform=new_affine * Affine.translation(-0.5,-0.5), height=dst_profile['height'] ,
                   width=dst_profile['width'])
'''
with rasterio.open(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EOBS/bio4_{str(year)}_25km.tif','w',**dst_profile) as dst:
    dst.write(bio4,1)

bio5  = np.ma.max(maxt, axis =0).filled(-9999)

with rasterio.open(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EOBS/bio5_{str(year)}_25km.tif','w',**dst_profile) as dst:
    dst.write(bio5,1)

print(perc[:,188,297])

quartperc = np.roll(perc, -1, axis =0) + perc + np.roll(perc, 1, axis = 0)
maxperc = np.max(quartperc, axis = 0)
print(quartperc[:,188,297])
print(maxperc[188,297])
argmaxperc = np.ma.argmax(quartperc, axis = 0)
avgtquart = (np.roll(avgt, -1, axis =0) + avgt + np.roll(avgt, 1, axis = 0) ) / 3.
print(avgtquart[:,188,297])


bio8 = np.take_along_axis(avgtquart, np.expand_dims(argmaxperc, axis=0), axis=0).squeeze(axis=0)
print(bio8[188,297])

with rasterio.open(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EOBS/bio8_{str(year)}_25km.tif','w',**dst_profile) as dst:
    dst.write(bio8.filled(-9999),1)

bio15 = np.ma.std(perc,axis=0)/(np.ma.mean(perc, axis=0)+1)*100 #+1 is to avoid strangr behavior when rain < 1.
print(bio15[188,297])
with rasterio.open(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EOBS/bio15_{str(year)}_25km.tif','w',**dst_profile) as dst:
    dst.write(bio15.filled(-9999),1)





