#### Expert-based Model: for the pollination potential provided by wild insect pollinators ####
####### This example includes CLC for year 2000 #####

library(rgdal)
library(maptools)
library(sp)
library(raster)
library(shapefiles)
library(abind)


homedir <- getwd() ## e.g. Users/myname/Pollination
setwd(homedir)

rd <- "/data/inca_vol1/input_data/crop_pollination/input_intermediate/annual_grid_datasets/" ## the source directory where Land Cover files are located; e.g. Users/myname/Pollination/CORINE_LC
id <- "Inputs"  ## e.g. Users/myname/Pollination/Inputs

td <- "/data/inca_vol1/input_data/crop_pollination/input_intermediate/EBM" ## e.g. Users/myname/Pollination/Intermediate
od <- "/data/inca_vol1/input_data/crop_pollination/input_final/EBM" ## e.g. Users/myname/Pollination/Outputs
sd <- "/data/inca_vol1/input_data/crop_pollination/input_intermediate/input_jrc/" ## e.g. Users/myname/Pollination/Scores
ai <- "Activity" ## e.g. Users/myname/Pollination/Activity  ## with the 'ActivityIndex' tif file (from April to September)
ms <- "/data/inca_vol1/input_data/crop_pollination/input_intermediate/INCA_extent/" ## e.g. Users/myname/Pollination/Mask with masks: EU28.tif, RdMj12_eu28.tif, EU28_0.tif (EU28 with landvalues set to 0)

setwd(id)
getwd() ##Input


MyRas <- list.files (path = rd, pattern = "\\.tif$", recursive = T)

abspath <- function(files)file.path(normalizePath(rd), files)


MyRas <- abspath(MyRas)
length(MyRas)

MyNam <- c('2000', '2006', '2012','2018')
#select year based on index
i = 4

rs <- raster(MyRas[i])                        ## Creating RasterLayer Objects
assign(paste('al',MyNam[i],'_'), rs) 

rs <- stack(rs)
dim(rs)
rs@layers
rs[[1]] ## Check name

## Reclassifying input Land Cover to extract forest types
## edge function in R

## Importing table with LC codes and scores

setwd(homedir)
setwd(sd)

scores <- read.csv("Scores_LC.csv", dec = ".", header = T,
                    stringsAsFactors=FALSE, fileEncoding="latin1" )
str(scores); dim (scores); names(scores)

### Preparing reclassification matrix to assign 1 to forest areas
m0 <- rbind(scores[, 1], scores [,3]) 
str(m0)
rclmat <- matrix (m0, ncol = 2, byrow = T)
rclmat <- rbind(rclmat,cbind(NA, 0))
dim(rclmat)
rclmat

rs[[1]] ##corine_2000_accounting_reclassified 

rct <- rs[[1]]
class(rct)

for_2000 <-reclassify(rct, rclmat)


## if necessary, set mask = eu28.tif AND replace 1 with 0 (eu28 - 1) and then multiply

## Exporting to GeoTiff
setwd("..")
setwd(td)

#proj4string(for_2000) <- CRS("+init=epsg:3035")
tif.name <- paste(td,'/for_',MyNam[i],'eu28', '.tif', sep="")
#writeRaster(for_2000, filename= tif.name, format="GTiff", datatype='INT1U',overwrite=TRUE)
print("end")

## Detecting boundaries (edges) in forest class
### 8 neighbours
#rs <- raster("for_2000eu28.tif")  
#myfile <- rs
myname <- paste('edges',MyNam[i], '.tif', sep="") 
edges <- boundaries(for_2000, type = 'inner', classes = T, directions = 8, asNA = F, filename= myname, format="GTiff", overwrite=TRUE) ### use classes = TRUE; asNA = FALSE
#edges2000 <- edges

#proj4string(edges2000) <- CRS("+init=epsg:3035")
#tif.name <- paste('edges2018', '.tif', sep="")
#writeRaster(edges2000, filename= tif.name, format="GTiff", datatype='INT1U',overwrite=TRUE)
print("end")

# 
# ### Defining edge type
# #setwd("..")
# #setwd(sd)
# #scores <- read.csv(file ="Scores_LC.csv", dec = ".", header = T )
# #str(scores); dim (scores); names(scores)
# 
# ### Scoring forest edges
# setwd("..")
# setwd(td)
# e.type00 <- raster("edges2012.tif")
# # 
# # 
# # setwd("..")
# # setwd(id)
# # 
# # MyRas <- list.files (pattern = "\\.tif$", recursive = T)
# # MyRas
# # length(MyRas)
# # 
# # MyNam <- c('al_1990', 'al_2000', 'al_2006', 'al_2012')
# # for (i in 1:length(MyRas)){
# #   rs <- raster(MyRas[i])                        ## Creating RasterLayer Objects
# #   assign(MyNam[i], rs)                          ## Assigning names to the RLOs
# # }
# # 
# # 
# # rs <- stack(al_2000, al_2006, al_2012)
# # dim(rs)
# # rs@layers
# # rs[[1]] ## Check name
# # 
# # 
# # setwd("..")
# # setwd (ms)
# # getwd()
# # msk <- raster("EU28.tif")  
# # 
# # 
# # setwd("..")
# # setwd (td)
# # getwd()
# # 
# # myfun <- function(x,y){return(x*y)}
# # 
# # 
# # r <- rs[[1]]
# # AL00_EU28 <- overlay(r, msk, fun = myfun)
# # AL00_EU28
# # plot(AL00_EU28)
# # 
# # myname <- "Al00_EU28.tif"
# # 
# # proj4string(AL00_EU28) <- CRS("+init=epsg:3035")
# # writeRaster(AL00_EU28, filename= myname, format="GTiff", datatype='INT1U',overwrite=TRUE)
# # 
# # 
# # 
# # ## Importing table with LC codes and scores
# # 
# # setwd("..")
# # setwd (sd)
# # getwd()
# # 
# # scores <- read.csv(file ="Scores_LC.csv", dec = ".", header = T )
# # str(scores); dim (scores); names(scores)
# # 
# #  
# 
# ### Preparing reclassification matrix to score LC for FA and NS
# 
# ### FA
# m0 <- rbind(scores[, 1], scores [,4]) 
# str(m0)
# rclmat <- matrix (m0, ncol = 2, byrow = T)
# dim(rclmat)
# rclmat
# FA <- rclmat
# 
# ### NS
# m0 <- rbind(scores[, 1], scores [,5]) 
# str(m0)
# rclmat <- matrix (m0, ncol = 2, byrow = T)
# dim(rclmat)
# rclmat
# NS <- rclmat
# 
# 
# setwd("..")
# setwd (td)
# getwd()
# 
# 
# #AL00_EU28 <- raster("AL00_EU28.tif")
# #plot(AL00_EU28) scoring stabdaard reclass
# AL00_EU28 <- al_2012
# 
# 
# al00.FA <- reclassify(AL00_EU28, FA, filename='FA_tmp.grd')
# al00.FA
# plot(al00.FA)
# myname <- "FA12_0.tif"
# #proj4string(al00.FA) <- CRS("+init=epsg:3035")
# writeRaster(al00.FA, filename= myname, format="GTiff", datatype='FLT4S',overwrite=TRUE)
# #file.remove(c('FA_tmp.grd', 'FA_tmp.gri'))
# al00.NS <- reclassify(AL00_EU28, NS)
# al00.NS
# 
# plot(al00.NS)
# myname <- "NS12_0.tif"
# #proj4string(al00.NS) <- CRS("+init=epsg:3035")
# writeRaster(al00.NS, filename= myname, format="GTiff", datatype='FLT4S',overwrite=TRUE)
# rm(al00.NS)
# print("done")
# 
# 
# ####  Scoring forest edges
# setwd("..")
# setwd (sd)
# getwd()
# 
# ### Preparing reclassification matrix to define edge type
# m0 <- rbind(scores[, 1], scores [,8])
# str(m0)
# rclmat <- matrix (m0, ncol = 2, byrow = T)
# dim(rclmat)
# rclmat
# EG <- rclmat
# 
# setwd("..")
# setwd (id)
# getwd()
# 
# al00.EG <- reclassify(AL00_EU28, EG)
# 
# setwd("..")
# setwd (td)
# getwd()
# 
# myname <- "edtyp12.tif"
# #proj4string(al00.EG) <- CRS("+init=epsg:3035")
# writeRaster(al00.EG, filename= myname, format="GTiff", datatype='FLT4S',overwrite=TRUE)
# e.type00 <- raster("edtyp12.tif")
# 
# ## FA
# setwd("..")
# setwd (sd)
# getwd()
# 
# 
# m0 <- rbind(scores[, 1], scores [,6]) 
# str(m0)
# rclmat <- matrix (m0, ncol = 2, byrow = T)
# dim(rclmat)
# rclmat
# FAed <- rclmat
# 
# 
# ## NS
# m0 <- rbind(scores[, 1], scores[,7]) 
# str(m0)
# rclmat <- matrix (m0, ncol = 2, byrow = T)
# dim(rclmat)
# rclmat
# NSed <- rclmat
# 
# 
# e00.FA <- reclassify(e.type00, FAed)
# e00.FA
# myname <- "FA12_edges.tif"
# #proj4string(e00.FA) <- CRS("+init=epsg:3035")
# writeRaster(e00.FA, filename= myname, format="GTiff", datatype='FLT4S',overwrite=TRUE)
# print("done")
# 
# e00.NS <- reclassify(e.type00, NSed)
# e00.NS
# myname <- "NS12_edges.tif"
# #proj4string(e00.NS) <- CRS("+init=epsg:3035")
# writeRaster(e00.NS, filename= myname, format="GTiff", datatype='FLT4S',overwrite=TRUE)
# 
# print("done")
# 
# ### Exporting to geotiff
# 
# setwd("..")
# setwd (td)
# getwd()
# 
# 
# 
# 
# 
# 
# ### Overlaying FA00_0.tif (where all forest areas = 0) to FA00_edges.tif (where forest edges are scored >0), and NS00_0.tif (where all forest areas = 0) to NS00_edges (where forest edges are scored >0)
# myfun <- function(x,y){return(x+y)}
# al00.FA <- raster("FA12_0.tif")
# e00.FA <- raster("FA12_edges.tif")
# x <-  al00.FA  ## LC scores for FA (with 0 for forests) ("FA00_0.tif")
# y <-  e00.FA  ## edge scores for FA ("FA00_edges.tif")
# FA00_1 <- overlay(x, y, fun = myfun)
# rm(x)
# rm(y)
# 
# myname <- "FA12_LCedg.tif"
# #proj4string(FA00_1) <- CRS("+init=epsg:3035")
# writeRaster(FA00_1, filename= myname, format="GTiff", datatype='FLT4S',overwrite=TRUE)
# 
# 
# 
# al00.NS <- raster("NS12_0.tif")
# e00.NS <- raster("NS12_edges.tif")
# x <-  al00.NS  ## LC scores for NS (with 0 for forests) ("NS00_0.tif")
# y <-  e00.NS  ## edge scores for NS ("NS00_edges.tif")
# NS00_1 <- overlay(x, y, fun = myfun)
# NS00_1
# plot(NS00_1)
# 
# myname <- "NS12_LCedg.tif"
# #proj4string(NS00_1) <- CRS("+init=epsg:3035")
# writeRaster(NS00_1, filename= myname, format="GTiff", datatype='FLT4S',overwrite=TRUE)
# 
# print("done")
# 
# 
# ### Updating the FA and NS scores to assign 0 to Major Roads
# 
# setwd("..")
# setwd(ms)
# ### Road file: "RdMj12_eu28.tif"
# 
# rd <- raster("/data/inca_vol1/input_data/crop_pollination/input_intermediate/OSM_INCA-AOI/OSM_main_roads_AOI-EU28_100m_EPSG3035.tif")
#  
# 
# setwd("..")
# setwd(td)
# 
# ns <- raster ("NS12_LCedg.tif")
# fa <- raster("FA12_LCedg.tif")
# 
# 
# myfun <- function(x,y){return(x*(1-y))}
# 
# x <-  fa  ## LC scores for FA, updated for the presence of forest and edge types ("FA00_LCedg.tif")
# y <-  subs(rd, data.frame(id=c(NA,0,1),v=c(0,0,1)))  ## major road file ("RdMj12_eu28.tif")
# writeRaster(y, filename= 'road', format="GTiff", datatype='INT1U',overwrite=TRUE)
# y <- raster('road')
# FA00_2 <- overlay(x, y, fun = myfun)
# plot(FA00_2)
# 
# print("done")
# rm(x)
# 
# #### Exporting to geotiff
# myname <- "FA12_LCedrd.tif"
# #proj4string(FA00_2) <- CRS("+init=epsg:3035")
# writeRaster(FA00_2, filename= myname, format="GTiff", datatype='FLT4S',overwrite=TRUE)
# 
# 
# x <-  ns  ## LC scores for FA, updated for the presence of forest and edge types ("FA00_LCedg.tif")
# y <-  rd  ## major road file ("RdMj12_eu28.tif")
# 
# NS00_2 <- overlay(x, y, fun = myfun)
# 
# myname <- "NS12_LCedrd.tif"
# #proj4string(NS00_2) <- CRS("+init=epsg:3035")
# writeRaster(NS00_2, filename= myname, format="GTiff", datatype='FLT4S',overwrite=TRUE)
# print("done")
# 
# 
# 
# #### Computing overall suitability for FA and NS (afterwards, two more updates are neeeded: 
# #### 1 to account for the distance to semi-natural areas, 
# #### 1 to account for activity index, which is influenced by temperature and irradiance)
# 
# getwd()
#  ## setwd("..")
#  ## setwd(ms)
# 
# FA <- raster("FA00_LCedrd.tif")
# NS <- raster("NS00_LCedrd.tif")
# 
# myfun <- function(x,y){return((x+y)/2)}
# 
# x <-  FA  ## LC scores for FA, updated for the presence of forest, edge types and Major roads
# y <-  NS  ## LC scores for NS, updated for the presence of forest, edge types and Major roads
# FANS <- overlay(x, y, fun = myfun)
# plot(FANS)
# 
# myname <- "FANS00.tif"
# proj4string(FANS) <- CRS("+init=epsg:3035")
# writeRaster(FANS, filename= myname, format="GTiff", datatype='FLT4S',overwrite=TRUE)
# print("done")
# 
# 
# #### Setting null values to FANS = 0 (for ED)
# 
# myfun <-function(x){
#   ifelse(x> 0, x, NA)}
# 
# FANS <- raster("FANS00.tif")
# FANSg0 <- calc(FANS,fun=myfun)
# plot(FANSg0)
# 
# myname <- "FANSg0_00.tif"
# proj4string(FANSg0) <- CRS("+init=epsg:3035")
# writeRaster(FANSg0, filename= myname, format="GTiff", datatype='FLT4S',overwrite=TRUE)
# print("done") 
# 
# 
# ### Computing Natural Logarithm of FANSg0
# myfun <- function(x){
#   ifelse(x >0, log(x), NA)}
# 
# lnFANSg0 <- calc(FANSg0, fun=myfun)
# plot(lnFANSg0)
# 
# myname <- "lnFANSg0_00.tif"
# proj4string(lnFANSg0) <- CRS("+init=epsg:3035")
# writeRaster(lnFANSg0, filename= myname, format="GTiff", datatype='FLT4S',overwrite=TRUE)
# print("done") 
# 
# 
# 
# ### Preparing reclassification matrix to assign 1 to all semi-(natural)areas and NA to the everything else.
# 
# setwd("..")
# setwd(sd)
# 
# scores <- read.csv(file ="Scores_LC.csv", dec = ".", header = T )
# str(scores); dim (scores); names(scores)
# 
# 
# m0 <- rbind(scores[, 1], scores [,9]) 
# str(m0)
# rclmat <- matrix (m0, ncol = 2, byrow = T)
# dim(rclmat)
# rclmat
# 
# 
# setwd("..")
# setwd(id)
# 
# r <- raster("Al00_EU28.tif") ##corine_2000_accounting_reclassified 
# 
# rct <- r
# class(rct)
# nat_00 <- reclassify(rct, rclmat)
# 
# 
# setwd("..")
# setwd(td)
# 
# 
# myname <- "nat00_eu28.tif"
# proj4string(nat_00) <- CRS("+init=epsg:3035")
# writeRaster(nat_00, filename= myname, format="GTiff", datatype='INT1U',overwrite=TRUE)
# print("done")
# 
# 
# 
# #### Calculating the distance of each cell from the closest 'natural area'
# 
# ### R does not handle this operation on the input file. It was calculated outside R, with a GIS software, and saved as GeoTiff
# 
# # tif.name <- paste('EDnat00_EU28', '.tif', sep = "")
# # dist_2000 <- distance(nat_00, filename='tif.name', doEdge=TRUE)
# # proj4string(dist_2000) <- CRS("+init=epsg:3035")
# 
# ### The distance map was calculated using Euclidean distance from 'natural areas' for each year, with a cut off distance of 20,000 m.
# ### The distance map D is the input to the distance decay model: y = exp(a + bD) + e
# ##### where a = intercept, b = distance decay, e = error term
# 
# 
# ############ Applying distance decay function FANSED = exp(ln(FANSg0) - 0.00046ED)
# 
# 
# ## The working directory is 'td'
# 
# ## ln(FANSg0) file: "lnFANSg0_00.tif"
# 
# ## ED file: "EDnat00_EU28.tif"
# 
# 
# lnFANSg0 <- raster("lnFANSg0_00.tif")
# ED <- raster("EDnat00_EU28.tif")
# 
# d <- 0.00046 
# 
# myfun <- function(x){d*x}
# 
# ddED <- calc(ED, myfun)
# plot(ddED)
# 
# myfun <- function(x,y){return(x-y)}
# x <- lnFANSg0
# y <- ddED
# 
# lnFANSg0ddED <- overlay(x, y, fun = myfun)
# plot(lnFANSg0ddED)
# 
# myname <- "lnFANSED_00.tif"
# proj4string(lnFANSg0ddED) <- CRS("+init=epsg:3035")
# writeRaster(lnFANSg0ddED, filename= myname, format="GTiff", datatype='FLT4S',overwrite=TRUE)
# print("done")
# 
# 
# ### Applying exponential function to backtransform distance decay
# 
# ## The working directory is 'td'
# 
# ### lnFANSg0ddED file: "lnFANSED_00.tif"
# 
# lnFANSg0ddED <- raster("lnFANSED_00.tif")
# 
# 
# myfun <- function(x){exp(x)}
# 
# r <- lnFANSg0ddED
# PS_0 <- calc(r, myfun) ### This file includes null values where suitability = 0 and will need to be modified
# plot(PS_0)
# 
# myname <- "PS00_0.tif"
# proj4string(PS_0) <- CRS("+init=epsg:3035")
# writeRaster(PS_0, filename= myname, format="GTiff", datatype='FLT4S',overwrite=TRUE)
# print("done")
# 
# 
# #######  Replacing NULL values with 0 (Null values correspond to areas where suitability for FA and NS is ZERO)
# ### EU_28_0.tif is the EU28 mask with landvalues set to 0
# 
# setwd("..")
# setwd(ms)
# eu28_0 <- raster("EU28_0.tif")
# 
# setwd("..")
# setwd(td)
# PS_0 <- raster ("PS00_0.tif")
# 
# rna <- reclassify(PS_0, cbind(NA, 0))
# plot(rna)
# 
# ### Masking EU28
# 
# myfun <- function(x, y){
#   return(x+y)}
# 
# x <- eu28_0
# y <- rna
# 
# PS_1 <- overlay(x,y, fun=myfun)
# plot(PS_1)
# 
# myname <- "PS00_1.tif"
# proj4string(PS_1) <- CRS("+init=epsg:3035")
# writeRaster(PS_1, filename= myname, format="GTiff", datatype='FLT4S',overwrite=TRUE)
# print("done")
# 
# 
# #### Accounting for differences in Activity Index
# 
# setwd("..")
# setwd(ai)
# AI <- raster("ai_4to9_00.tif")
# 
# setwd("..")
# setwd(td)
# PS_1 <- raster("PS00_1.tif")
# 
# myfun <- function(x, y){
#   return(x*y)}
# 
# x <- PS_1
# y <- AI
# 
# PSAI <- overlay(x,y, fun=myfun)
# plot(PSAI)
# 
# setwd("..")
# setwd(od)
# 
# myname <- "PSAI_00.tif"
# proj4string(PSAI) <- CRS("+init=epsg:3035")
# writeRaster(PSAI, filename= myname, format="GTiff", datatype='FLT4S',overwrite=TRUE)
# print("done")

