#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2022 European Union.
#
# The tool was developed with the contribution of the Joint Research Centre of the European Commission.
#
# This program is free software: you can redistribute it and/or modify it under the terms of the European Union Public
# Licence, either version 1.2 of the License, or (at your option) any later version.
# You may not use this work except in compliance with the Licence.
#
# You may obtain a copy of the Licence at: https://joinup.ec.europa.eu/collection/eupl/eupl-guidelines-faq-infographics
#
# Unless required by applicable law or agreed to in writing, software distributed under the Licence is distributed on
# an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the Licence for the specific language governing permissions and limitations under the Licence.


"""
SYNOPSIS
    run_EBM_calculation.py [-v,--verbose] [-h,--help] [--version]

DESCRIPTION
    This snippet is for running the EBM model in preparation of the crop pollination model run.

PREREQUISITES
    Python => 3.7.0

EXAMPLES
    python -m inca.croppollination.preprocessing.run_EBM_calculation -i <file_path> -o <folder_path> -r <ref_file>
    OR
    python3 run_EBM_calculation.py -i <file_path> -o <folder_path> -r <ref_file>

EXIT STATUS
    Ctrl-C  =  cancel the script at any time

AUTHOR
    Dr. Marcel Buchhorn <marcel.buchhorn@vito.be>

LICENSE
    This script is property of VITO. Copyrights or restrictions may apply.

VERSION
    0.1 (2021-11-18)
"""

''' STILL a dummy '''

import rasterio
import math
import numpy as np
from inca.common.classification import reclassification, CSV_2_dict
import inca.common.geoprocessing as geo

year = 2018
print(year)
path_scores ='/data/inca_vol1/input_data/crop_pollination/input_intermediate/input_jrc/Scores_LC.csv'
scores_forest = CSV_2_dict(path_scores, 'Value','Forest')
scores_FA = CSV_2_dict(path_scores, 'Value','FA')
scores_NS = CSV_2_dict(path_scores, 'Value','NS')
scores_Edge_FA = CSV_2_dict(path_scores, 'Value','Edge_FA')
scores_Edge_NS = CSV_2_dict(path_scores, 'Value','Edge_NS')
scores_Edge_type = CSV_2_dict(path_scores, 'Value','Edge_type')
scores_Nat_area = CSV_2_dict(path_scores, 'Value','Nat_area')

#First generate nat distance map
nat_dist_map = f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EBM/natdist_{str(year)}.tif'
'''
with rasterio.open(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/annual_grid_datasets/Grid_crop-pollination_{str(year)}.tif') as src:
    grid = src.read()
    profile = src.profile
    nat, dClasses = reclassification(grid, scores_Nat_area, src.nodata, src.nodata)
    with rasterio.open(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EBM/nat_{str(year)}.tif','w',**profile) as dst:
        dst.write(nat)
    nat = None
'''
srcfile = f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EBM/nat_{str(year)}.tif'

cmd = f"gdal_proximity.py -srcband 1 -values 1 -maxdist 20000 -nodata 25500 -distunits GEO -of GTiff -co COMPRESS=DEFLATE -co BIGTIFF=YES -co TILED=YES {srcfile} {nat_dist_map} "

try:
    print('')
    #subprocess.check_call(cmd, shell=True)
except Exception:
    raise OSError('Could not run gdal_proximity')

#create FA and NS,  + correct for edges and roads
path_out = f'/data/inca_vol1/input_data/crop_pollination/input_final/EBM/EBM_crop-pollination_{str(year)}.tif'
block_shape =(2048, 2048)
with rasterio.open(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/annual_grid_datasets/Grid_crop-pollination_{str(year)}.tif') as src, \
    rasterio.open('/data/inca_vol1/input_data/crop_pollination/input_intermediate/OSM_INCA-AOI/OSM_main_roads_AOI-EU28_100m_EPSG3035.tif') as osm, \
    rasterio.open(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EBM/ai_{str(year)}_INCA_100m_EPSG3035.tif') as ai, \
    rasterio.open(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EBM/edges{str(year)}.tif') as edge_src, \
    rasterio.open(nat_dist_map) as nat_dist:
    # create new file
    profile_dst = ai.profile
    profile_dst['nodata'] = -1
    profile_dst['dtype'] = np.float64
    profile_dst['BIGTIFF'] = 'YES'
    with rasterio.open(path_out, 'w', **profile_dst) as dst:
        # set tags
        #dst.update_tags(**tags)
        if True:
            max_counter = math.ceil(profile_dst['height'] * 1.0 / block_shape[0]) * math.ceil(
                profile_dst['width'] * 1.0 / block_shape[1])
            counter = 1
            pstatus = 0

        # now iterate over the blocks of src files and process
        for index, window in geo.block_window_generator(block_shape, profile_dst['height'], profile_dst['width']):
            grid = src.read(window = window)

            Forests, dClasses = reclassification(grid, scores_forest, src.nodata, src.nodata)
            FA, dClasses = reclassification(grid, scores_FA, src.nodata, src.nodata, outputtype=np.float64)

            NS, dClasses = reclassification(grid, scores_NS, src.nodata, src.nodata, outputtype=np.float64)

            # Find edges:
            # will not be done athe moment read in from R

            edges = edge_src.read(window = window)

            edge_type = np.zeros_like(grid)
            edge_type[edges != 0] = grid[edges != 0]
            FA_edge, dClasses = reclassification(edge_type, scores_Edge_FA, src.nodata, src.nodata, outputtype=np.float64)

            NS_edge, dClasses = reclassification(edge_type, scores_Edge_NS, src.nodata, src.nodata, outputtype=np.float64)


            FA_LCedge = FA + FA_edge
            NS_LCedge = NS + NS_edge

            roads = osm.read(window=window)
            FA_LCedge[roads == 1] = 0
            NS_LCedge[roads == 1] = 0

            FANS = (NS_LCedge + FA_LCedge) / 2.
            #logfans = np.log(FANS)

            NODATA = -1
            dist = nat_dist.read(window=window) #(is in 1m )

            PS_0 = FANS / np.exp(0.00046*dist)
            # Clear from NA's
            PS_0[FANS==0] = 0
            PS_0[grid == 0] = NODATA

            AI = ai.read(window=window)

            ebm = AI*PS_0/100.
            ebm[AI<0] = 0
            dst.write(ebm.astype(np.float64),window=window)
            if True:
                progress = (counter / max_counter) * 100
                if (progress - pstatus) >= 1:
                    pstatus = int(progress)
                    print('{} %'.format(pstatus))
                counter += 1












