#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2022 European Union.
#
# The tool was developed with the contribution of the Joint Research Centre of the European Commission.
#
# This program is free software: you can redistribute it and/or modify it under the terms of the European Union Public
# Licence, either version 1.2 of the License, or (at your option) any later version.
# You may not use this work except in compliance with the Licence.
#
# You may obtain a copy of the Licence at: https://joinup.ec.europa.eu/collection/eupl/eupl-guidelines-faq-infographics
#
# Unless required by applicable law or agreed to in writing, software distributed under the Licence is distributed on
# an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the Licence for the specific language governing permissions and limitations under the Licence.


"""
SYNOPSIS
    run_SDM_calculation.py [-v,--verbose] [-h,--help] [--version]

DESCRIPTION
    This snippet is for running the SDM model in preparation of the crop pollination model run (full run).

PREREQUISITES
    Python => 3.7.0

EXAMPLES
    python -m inca.croppollination.preprocessing.run_SDM_calculation -i <file_path> -o <folder_path> -r <ref_file>
    OR
    python3 run_SDM_calculation.py -i <file_path> -o <folder_path> -r <ref_file>

EXIT STATUS
    Ctrl-C  =  cancel the script at any time

AUTHOR
    Dr. Marcel Buchhorn <marcel.buchhorn@vito.be>

LICENSE
    This script is property of VITO. Copyrights or restrictions may apply.

VERSION
    0.1 (2021-11-18)
"""
import glob
import sys

import numpy as np
import rasterio
from rasterio.windows import from_bounds
import pandas as pd
import subprocess

''' STILL a dummy '''

year =  2018
print(year)

def average_chunk(x, chunk_size, func = np.average, axis=-1):
    if axis == -1:
        shape = (x.shape[0] // chunk_size, chunk_size, x.shape[1] // chunk_size, chunk_size)
        x = x.reshape(shape)
        return func(x, axis=(1, -1))

    else:
        shape = x.shape
        if axis < 0:
            axis += x.ndim
        shape = shape[:axis] + (-1, chunk_size) + shape[axis + 1:]
        x = x.reshape(shape)
        return func(x, axis=axis + 1)

#read in preprocessed bio4, bio5, bio8, bio15
print('reading in biovars')
with rasterio.open(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EOBS/bio4_2012_25km_INCA_10000m_EPSG3035.tif') as src:
    bio04 =src.read(1)
    left, bottom, right, top = src.bounds
    dst_profile = src.profile
with rasterio.open(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EOBS/bio5_2012_25km_INCA_10000m_EPSG3035.tif') as src:
    bio05 =src.read(1)
with rasterio.open(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EOBS/bio8_2012_25km_INCA_10000m_EPSG3035.tif') as src:
    bio08 =src.read(1)
with rasterio.open(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EOBS/bio15_2012_25km_INCA_10000m_EPSG3035.tif') as src:
    bio15 =src.read(1)


print('reading in CCI')
with rasterio.open(f'/data/inca_vol1/input_data/soil_retention/input_raw/land-cover_CORINE/CLCACC/CLC{str(year)}ACC_V2018_20.tif') as src:
    window = from_bounds(left, bottom, right, top, transform=src.transform)
    print(left,bottom,right,top)
    print(window)
    print(src.bounds)
    grid = src.read(1,window= window,boundless = True, fill_value=0)


print("calculating lu's")
#lu_DUF
DUF = np.zeros_like(grid)
DUF[grid==112] = 1
lu_DUF = average_chunk(DUF,100)*100
DUF = None
#lu_GUS
GUS = np.zeros_like(grid)
GUS[(grid==141) | (grid == 142)] = 1
lu_GUS = average_chunk(GUS,100)*100
GUS = None

#lu_AL
AL = np.zeros_like(grid)
AL[(grid==211) | (grid == 212) | (grid == 213)] = 1
lu_AL = average_chunk(AL,100)*100
AL = None

#lu_PC
PC = np.zeros_like(grid)
PC[(grid==221) | (grid == 222) | (grid == 223)] = 1
lu_PC = average_chunk(PC,100)*100
PC = None

#lu_PA
PA = np.zeros_like(grid)
PA[(grid==231) ] = 1
lu_PA = average_chunk(PA,100)*100
PA = None

#lu_HAG
HAG = np.zeros_like(grid)
HAG[(grid==241) | (grid == 242) | (grid == 244)] = 1
lu_HAG = average_chunk(HAG,100)*100
HAG = None

#lu_AGNV
AGNV = np.zeros_like(grid)
AGNV[(grid==243) ] = 1
lu_AGNV = average_chunk(AGNV,100)*100
AGNV = None

#lu_BF
BF = np.zeros_like(grid)
BF[(grid==311) ] = 1
lu_BF = average_chunk(BF,100)*100
BF = None

#lu_CF
CF = np.zeros_like(grid)
CF[(grid==312) ] = 1
lu_CF = average_chunk(CF,100)*100
CF = None

#lu_MF
MF = np.zeros_like(grid)
MF[(grid==313) ] = 1
lu_MF = average_chunk(MF,100)*100
MF = None

#lu_NG
NG = np.zeros_like(grid)
NG[(grid==321) ] = 1
lu_NG = average_chunk(NG,100)*100
NG = None

#lu_SMH
SMH = np.zeros_like(grid)
SMH[(grid==322) | (grid==323) | (grid==324) ] = 1
lu_SMH = average_chunk(SMH,100)*100
SMH = None

#lu_BDSV
BDSV = np.zeros_like(grid)
BDSV[(grid==331) | (grid==333)  ] = 1
lu_BDSV = average_chunk(BDSV,100)*100
BDSV = None

#lu_IW
IW = np.zeros_like(grid)
IW[(grid==411) | (grid==412) ] = 1
lu_IW = average_chunk(IW,100)*100
IW = None

#lu_BW
BW = np.zeros_like(grid)
BW[(grid==421)] = 1
lu_BW = average_chunk(BW,100)*100
BW = None

#lu_IWB
IWB = np.zeros_like(grid)
IWB[(grid==511) | (grid==512)  ] = 1
lu_IWB = average_chunk(IWB,100)*100
IWB = None
grid = None

#Converting distance from snd_km
print('processing snd_km')
with rasterio.open(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EBM/natdist_{year}.tif') as natdist_src:
    window = from_bounds(left, bottom, right, top, transform=natdist_src.transform)
    print(window)
    snd = natdist_src.read(1,window= window,boundless = True, fill_value=25500)/1000. #(is in m should be in km)




snd_km = average_chunk(snd, 100)

snd = None
#Getting DEM info
print('processing dem')
with rasterio.open(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EOBS/dt1_dem_INCA_INCA_10000m_EPSG3035_mode.tif') as dem_src:
    window = from_bounds(left, bottom, right, top, transform=dem_src.transform)
    elmode = dem_src.read(1, window=window)
    print(window)
    elmode[elmode == -32767] = 0



# create output for running SDM model
rows = range(int(left)+5000, int(right), 10000) #samples should be taken cell centered and 10km res
columns = range(int(top)-5000, int(bottom), -10000)

#element to process:
variables = [bio04,bio05,bio08,bio15,elmode,lu_AGNV,lu_AL,lu_BDSV,lu_BF,lu_BW,lu_CF,lu_DUF,lu_GUS,lu_HAG,lu_IW,lu_IWB,lu_MF,lu_NG,lu_PA,lu_PC,lu_SMH,snd_km]
varText = ["bio04","bio05","bio08","bio15","elmode","lu_AGNV","lu_AL","lu_BDSV","lu_BF","lu_BW","lu_CF","lu_DUF","lu_GUS","lu_HAG","lu_IW","lu_IWB","lu_MF","lu_NG","lu_PA","lu_PC","lu_SMH","snd_km"]

total_df = pd.DataFrame(index= pd.MultiIndex.from_product([['AOI'],rows,columns], names=['Species', 'X' ,'Y']))
for idx, element in  enumerate(variables):
    element = np.transpose(element)
    df = pd.DataFrame(element, index = pd.MultiIndex.from_product([['AOI'],rows]), columns=columns)
    df = df.stack().rename(varText[idx])
    df.index.names = ['Species', 'X' ,'Y']

    total_df = total_df.join(df)

total_df[total_df==-9999]= None
total_df.dropna(inplace=True)
print(total_df.max(numeric_only=True))
print(total_df.min(numeric_only=True))
swd_file = f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EOBS/AOI_swd_{str(year)}.csv'
total_df.to_csv(swd_file)







