#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2022 European Union.
#
# The tool was developed with the contribution of the Joint Research Centre of the European Commission.
#
# This program is free software: you can redistribute it and/or modify it under the terms of the European Union Public
# Licence, either version 1.2 of the License, or (at your option) any later version.
# You may not use this work except in compliance with the Licence.
#
# You may obtain a copy of the Licence at: https://joinup.ec.europa.eu/collection/eupl/eupl-guidelines-faq-infographics
#
# Unless required by applicable law or agreed to in writing, software distributed under the Licence is distributed on
# an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the Licence for the specific language governing permissions and limitations under the Licence.


"""
SYNOPSIS
    run_SDM_projection.py [-v,--verbose] [-h,--help] [--version]

DESCRIPTION
    This snippet is for projecting the SDM model with existing lambda files to later time steps
    in preparation of the crop pollination model run.

PREREQUISITES
    Python => 3.7.0

EXAMPLES
    python -m inca.croppollination.preprocessing.run_SDM_projection -i <file_path> -o <folder_path> -r <ref_file>
    OR
    python3 run_SDM_projection.py -i <file_path> -o <folder_path> -r <ref_file>

EXIT STATUS
    Ctrl-C  =  cancel the script at any time

AUTHOR
    Dr. Marcel Buchhorn <marcel.buchhorn@vito.be>

LICENSE
    This script is property of VITO. Copyrights or restrictions may apply.

VERSION
    0.1 (2021-11-18)
"""

''' STILL a dummy '''
import pandas as pd
import numpy as np
import rasterio
year = 2018

apo = pd.read_csv(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/SDM/{str(year)}_average_probability_occurence.csv').set_axis(['Col_1', 'apo'], axis=1)
base_list  = pd.read_csv(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EOBS/AOI_swd_{str(year)}.csv',usecols = ['X','Y'])

base_list['apo'] = apo.apo

base_list.set_index(['X','Y'], inplace = True)
print(np.max(base_list))
with rasterio.open(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EOBS/bio4_2012_25km_INCA_10000m_EPSG3035.tif') as src:
    profile = src.profile
    left, bottom, right, top = src.bounds

rows = range(int(left)+5000, int(right), 10000) #samples should be taken cell centered and 10km res
columns = range(int(bottom)+5000, int(top), 10000)

input_pd = pd.DataFrame(index= pd.MultiIndex.from_product([rows,columns], names=['X' ,'Y']))
combi = input_pd.join(base_list)
combi = combi.unstack(level=0)
combi  = combi.sort_index(ascending=False)
print(combi)
with rasterio.open(f'/data/inca_vol1/input_data/crop_pollination/input_final/SDM/{str(year)}_aggminth.tif','w',**profile) as dst:
    dst.write(combi,1)

