#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2022 European Union.
#
# The tool was developed with the contribution of the Joint Research Centre of the European Commission.
#
# This program is free software: you can redistribute it and/or modify it under the terms of the European Union Public
# Licence, either version 1.2 of the License, or (at your option) any later version.
# You may not use this work except in compliance with the Licence.
#
# You may obtain a copy of the Licence at: https://joinup.ec.europa.eu/collection/eupl/eupl-guidelines-faq-infographics
#
# Unless required by applicable law or agreed to in writing, software distributed under the Licence is distributed on
# an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the Licence for the specific language governing permissions and limitations under the Licence.


"""
SYNOPSIS
    run_EBM_calculation.py [-v,--verbose] [-h,--help] [--version]

DESCRIPTION
    This snippet is for running the EBM model in preparation of the crop pollination model run.

PREREQUISITES
    Python => 3.7.0

EXAMPLES
    python -m inca.croppollination.preprocessing.run_EBM_calculation -i <file_path> -o <folder_path> -r <ref_file>
    OR
    python3 run_EBM_calculation.py -i <file_path> -o <folder_path> -r <ref_file>

EXIT STATUS
    Ctrl-C  =  cancel the script at any time

AUTHOR
    Dr. Marcel Buchhorn <marcel.buchhorn@vito.be>

LICENSE
    This script is property of VITO. Copyrights or restrictions may apply.

VERSION
    0.1 (2021-11-18)
"""

''' STILL a dummy '''

import pandas as pd
import geopandas as gpd
import os
import subprocess

def rasterize(gdb, path_out,column_name, nodata_value=0, dtype='Float32'):
    """ this function rasterize a given geopandas dataframe with a specified Column_name
         optional also give nodata value and dtype (dtype as string)"""
    # write the geodatabase to file can't use the general / not general enough

    temp_out = os.path.join(os.path.dirname(path_out), 'vector_{}.shp'.format(os.path.basename(path_out).split('.')[0]))

    cmd = 'gdal_rasterize -a {} -l {} -init {} -a_nodata {} -co COMPRESS=LZW -ot {} -te {} {} {} {} -tr {} {} {} {}'.format(
        column_name,
        os.path.basename(temp_out).split('.shp')[0],
        nodata_value,
        nodata_value,
        dtype,
        "2525000.0",
        "1386700.0",
        "6575000.0",
        "5466700.0",
        25000.0,
        25000.0,
        temp_out,
        path_out)


    try:

        gdb.to_file(temp_out)
        subprocess.check_call(cmd, shell=True)
    except Exception:
        raise OSError('Could not rasterize needed vector file')





#input for AI
year = 2006
Monthly_Rad = pd.read_csv(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/Agri4Cast_preprocessed/unique_meanrad_{str(year-1)}{str(year+1)}_monthly.csv')
Monthly_MeanT = pd.read_csv(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/Agri4Cast_preprocessed/unique_meantemp_{str(year-1)}{str(year+1)}_monthly.csv')
Monthly_daylength = pd.read_csv('/data/inca_vol1/input_data/crop_pollination/input_intermediate/Agri4Cast_preprocessed/avg_daylength.csv')
names = ['Month_Code','CODE_NO']
Monthly_Rad.index = pd.MultiIndex.from_frame(Monthly_Rad.iloc[:, 0:2], names=names)
Monthly_MeanT.index = pd.MultiIndex.from_frame(Monthly_MeanT.iloc[:, 0:2], names=names)
Monthly_daylength.index = pd.MultiIndex.from_frame(Monthly_daylength.iloc[:, 0:2], names=names)

all_info = pd.concat([Monthly_Rad, Monthly_MeanT, Monthly_daylength], axis=1)

all_info['Solar_irr'] = all_info['AVG_RAD']/ all_info['avg_daylength']*10/36 # (kJ / m2 / day) / (hours_light / day) -> transpose to W:  kJ/H -> 1000/3600 [W]
all_info['BlackGlobe'] = -0.62 + 1.027 * all_info['TEMPERATURE_AVG'] + 0.006*all_info['Solar_irr']
all_info['AI'] = -39.3 + 4.01*all_info['BlackGlobe']
DownSizing = all_info.loc[([4,5,6,7,8,9], slice(None)), : ]
DownSizing = DownSizing['AI']
Aggregate = DownSizing.groupby(level=1).mean()



gdf = gpd.read_file('/data/inca_vol1/input_data/crop_pollination/input_raw/Agri4Cast/grid_25km.shp')
gdf = gdf.merge(Aggregate, how='left', left_on='Grid_Code', right_on='CODE_NO')


rasterize(gdf, f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EBM/ai_{str(year)}_25km.tif', 'AI')






