import os
import pandas as pd
import numpy as np
import matplotlib
matplotlib.use('Qt5Agg')
from matplotlib import pyplot as plt
import brewer2mpl

# create graphs
# -------------

PATH = r'C:\Dfold\inca2021series\output\tourism2021_fullseries_{}'
YEARS = [2000,2006,2012,2018,2021]
SAVE = True
# import SUT csv's

supply_dict = dict()
use_dict = dict()

for year in YEARS:
    file2read = 'tourism_report_SUT_amountOvernightStays_{}.xlsx'.format(year)
    path2readfrom = os.path.join(PATH.format(year),"SUTs",file2read)
    Useloaded = pd.read_excel(path2readfrom,skiprows=2,sheet_name='Use')
    Supplyloaded = pd.read_excel(path2readfrom,skiprows=2,sheet_name='Supply')
    Useloaded['Households final consumption'][Useloaded['Unnamed: 1'] == 'all regions'] = np.nansum(Useloaded['Households final consumption'])
    Useloaded['Exports'][Useloaded['Unnamed: 1'] == 'all regions'] = np.nansum(Useloaded['Exports'])
    Useloaded["total"] =  Useloaded["Households final consumption"] +  Useloaded["Exports"]
    use_year = Useloaded["total"]
    use_region = Useloaded["Unnamed: 1"]
    supply_year = Supplyloaded.loc[0,:]
    cols2fill = supply_year.index
    cols2fill = cols2fill[2:]
    for col in cols2fill:
        Supplyloaded[col][Supplyloaded["Unnamed: 1"] == "all regions"] = np.nansum(Supplyloaded[col])
    total = np.nansum(Supplyloaded.loc[0, :][3:])
    Supplyloaded["total"][Supplyloaded["Unnamed: 1"] == "all regions"] = total
    supply_year = Supplyloaded.loc[0, :]
    supply_year = supply_year[2:]
    supply_eco = cols2fill

    use_dict[year] = use_year
    supply_dict[year] = supply_year

use_df = pd.DataFrame.from_dict(use_dict)
use_df = use_df.set_index(pd.Index(list(use_region)))

supply_df = pd.DataFrame.from_dict(supply_dict)
supply_df.set_index(supply_eco)

# devide by 1 million because values are very large
use_df = use_df/1000000
supply_df = supply_df/1000000

# plot bar-graphs

# colors
yearcol = brewer2mpl.get_map("Set1","qualitative",5).mpl_colors


# use
ax = use_df.iloc[1:,:].plot.bar(rot=0,figsize=(14,5),color=yearcol,width=0.8)
plt.title("Tourism 2021-series: Use - EU27")
plt.ylabel("number of overnight stays (million)")
ax.yaxis.set_major_locator(plt.MaxNLocator(13))
plt.grid(axis='y', linestyle='dotted')
if SAVE:
    plt.savefig('use.png', dpi=600, format='png')
else:
    plt.show()
plt.close()

ax = use_df.loc["all regions",:].plot.bar(rot=0,figsize=(7,4),color=yearcol)
plt.title("Tourism 2021-series: Use, summed for all EU27 member states")
plt.ylabel("number of overnight stays (million)")
if SAVE:
    plt.savefig('use_total.png', dpi=600, format='png')
else:
    plt.show()

#supply

ax = supply_df.iloc[1:,:].plot.bar(rot=0,figsize=(16,5),color=yearcol)
plt.title("Tourism 2021-series: Supply")
plt.ylabel("number of overnight stays (million)")
xtick_labels = [label.get_text() for label in ax.get_xticklabels()]
plt.gca().set_xticklabels([label.replace(' ', '\n') for label in xtick_labels])
plt.xticks(fontsize=7)
ax.figure.subplots_adjust(bottom=0.2)
ax.yaxis.set_major_locator(plt.MaxNLocator(13))
plt.grid(axis='y', linestyle='dotted')
if SAVE:
    plt.savefig('supply.png', dpi=900, format='png')
else:
    plt.show()

ax = supply_df.loc["total",:].plot.bar(rot=0,figsize=(7,4),color=yearcol)
plt.title("Tourism 2021-series: Supply, summed for all Ecosystems")
plt.ylabel("number of overnight stays (million)")
if SAVE:
    plt.savefig('supply_total.png', dpi=600, format='png')
else:
    plt.show()
print('done')



# sanity checks
# -------------

