import pandas as pd
import geopandas as gpd
import numpy as np
import os

current_path = os.getcwd()

# nuts data shapes
nuts2021 = r"C:\repos\inca\src\inca\data\NUTS_RG_03M_2021_3035_LEVL012_EU2020.gpkg"
nuts2013 = r"C:\repos\inca_geopandas\NUTS_RG_03M_2013_3035.shp\NUTS_RG_03M_2013_3035.shp"
nuts2016 = r"C:\repos\inca_geopandas\NUTS_RG_03M_2016_3035.shp\NUTS_RG_03M_2016_3035.shp"

nutsshapes = {
    2013: nuts2013,
    2016: nuts2016,
    2021: nuts2021
}

shapefiles = dict()
for year, path in nutsshapes.items():
    shapefiles[year] = gpd.read_file(path)

RO21 = {2006:
            {"foreign": 191046,
             "reporting":1408011},
        2012:
            {"foreign": 189699,
             "reporting":1486703
},
        2018:
            {"foreign": 281156,
             "reporting":2127348},
        2021:
            {"foreign": 92678,
             "reporting":1893732},
        }


years2process_unproc = [2000,2006,2012,2018,2021]
file2process = "data_nuts_overnight/Voorbereiding inputtabellen {} nin2.xlsx"

data_unproc_dict = dict()
for year in years2process_unproc:
    if year != 2021:
        file2load = file2process.format(year)
    else:
        file2load = "data_nuts_overnight/Voorbereiding inputtabellen 2021 nin2d.xlsx"
    data_unproc = pd.read_excel(file2load,
                         sheet_name="Basistabel",
                         skiprows=4)
    if year in [2000,2006,2012]:
        data_unproc = data_unproc.iloc[:, :-2]

        # rename cols
        data_unproc.columns=["nutscode",
                    "nutsname",
                    "foreign",
                    "reporting",
                    "total"]

    elif year == 2018:
        data_unproc = data_unproc.iloc[:, [0,1,2,4,6]]

        # rename cols
        data_unproc.columns=["nutscode",
                    "nutsname",
                    "foreign",
                    "total",
                    "reporting"]
    elif year == 2021:
        data_unproc = data_unproc.iloc[:, [1,2,3,7,11]]

        # rename cols
        data_unproc.columns = ["nutscode",
                        "nutsname",
                        "foreign",
                        "reporting",
                        "total"]
    data_unproc_dict[year] = data_unproc

# read the data
# 2000
# ----
years = [2000,2006,2012,2018,2021]
data_dict = dict()
for year in years:
    data_dict[year] = pd.read_csv(os.path.join(current_path,"final_csvs_overnightstays","overnight_stays_{}.csv".format(year)))


# countries on NUTS0 level
# ////////////////////////
excel_data2000 = data_dict[2000]
raw_data = data_unproc_dict[2000]
list_nuts0_to_correct = ["BG","HU","RO","IE","SK","SI","LT"]
years_to_check = [2006,2012,2018,2021]
types = ['Foreign country', 'Reporting country']

# loop over MS
for ms in list_nuts0_to_correct:
    # grab all NUTS2 per MS
    ms_data = excel_data2000[excel_data2000['DataAreaCode'].str.contains(ms)]

    # split up reporting/ foreign
    for type_rep in types:
        ms_data_type =  ms_data[ms_data["CountryOfResidence"] == type_rep]

        # for each year, get the % of spread of visits
        for yr in years_to_check:
            data = data_dict[yr] # read data
            ms_data_yr = data[data['DataAreaCode'].str.contains(ms)] # get the ms data
            ms_data_yr = ms_data_yr[ms_data_yr["CountryOfResidence"] == type_rep]

            if ms_data_yr["OvernightStays"].sum() == 0:
                ms_data_type["spread_{}".format(yr)] = np.nan
            else:
                ms_data_type["spread_{}".format(yr)] = ms_data_yr["OvernightStays"]/ms_data_yr["OvernightStays"].sum() # calc fractions


        # make the average of this spread
        ms_data_type["avg_spread"] =  ms_data_type[['spread_2006', 'spread_2012', 'spread_2012', 'spread_2021']].mean(axis=1)

        # apply this average spread to total nuts0 for the missing year
        total_NUTS2_type_report = raw_data[raw_data["nutscode"] == ms]
        if type_rep == "Foreign country":
            type_b = "foreign"
        else:
            type_b = "reporting"
        ms_data_type["OvernightStaysCorr"] = total_NUTS2_type_report[type_b].values[0] * ms_data_type["avg_spread"]
        joinList = ms_data_type["Unnamed: 0"]

        excel_data2000.loc[excel_data2000['Unnamed: 0'].isin(joinList),"OvernightStays"] = ms_data_type.loc[ms_data_type['Unnamed: 0'].isin(joinList), "OvernightStaysCorr"]

        print('a')

# do the same for finland, but using NUTS1 instead of nuts2
#///////////////////////////////////////////////////////////
def get_fractions_and_downscale_nuts(cntry_codes,parent_nuts_code,years_to_check,raw_data_curr_year,raw_data_other_yr,df_to_fill):
    """

    """

    # loop over codes - get fraction in other years
    for type_rep in types:
        for cntry_code in cntry_codes:
            ratios = []
            for yr in years_to_check:
                data_raw_yr = raw_data_other_yr[yr]
                if type_rep == "Foreign country":
                    type_b = "foreign"
                else:
                    type_b = "reporting"
                NUTS1 = data_raw_yr[type_b][data_raw_yr["nutscode"] == parent_nuts_code].values[0]
                NUTS2 = data_raw_yr[type_b][data_raw_yr["nutscode"] == cntry_code].values[0]
                ratio = NUTS2 / NUTS1
                ratios.append(ratio)

            # make average
            mean_years = np.mean(ratios)

            # spread out NUTS parent (NUTS1 or 2)  with the fractions
            nuts1_2000 = raw_data_curr_year[type_b][raw_data["nutscode"] == parent_nuts_code].values[0]
            cntry_corr = mean_years * nuts1_2000
            df_to_fill.loc[(excel_data2000['DataAreaCode'] == cntry_code) & (
                        df_to_fill['CountryOfResidence'] == type_rep), "OvernightStays"] = cntry_corr

    return df_to_fill

# Finland Codes to check, NUTS1 available (FI1)
fin_codes = ["FI1B","FI1C","FI1D"]
fin_source = "FI1"

excel_data2000 = get_fractions_and_downscale_nuts(cntry_codes=fin_codes,
                                                  parent_nuts_code=fin_source,
                                                  years_to_check=years_to_check,
                                                  raw_data_curr_year=data_unproc_dict[2000],
                                                  raw_data_other_yr=data_unproc_dict,
                                                  df_to_fill=excel_data2000)


# sporadic missing data
# //////////////////////

# germany data available for NUTS1: DEB
# get fractions for these:
ger_codes = ["DEB1","DEB2","DEB3"]
ger_source = "DEB"

excel_data2000 = get_fractions_and_downscale_nuts(cntry_codes=ger_codes,
                                                  parent_nuts_code=ger_source,
                                                  years_to_check=years_to_check,
                                                  raw_data_curr_year=data_unproc_dict[2000],
                                                  raw_data_other_yr=data_unproc_dict,
                                                  df_to_fill=excel_data2000)

# Italy NUTS1 also missing, get fractions for these for NUTS0 in other year:
it_codes = ["ITH1", "ITH2","ITH3","ITH4","ITH5","ITI1","ITI2","ITI3","ITI4"]
it_source = "IT"

excel_data2000 = get_fractions_and_downscale_nuts(cntry_codes=it_codes,
                                                  parent_nuts_code=it_source,
                                                  years_to_check=years_to_check,
                                                  raw_data_curr_year=data_unproc_dict[2000],
                                                  raw_data_other_yr=data_unproc_dict,
                                                  df_to_fill=excel_data2000)

# Greece NUTS1 also missing, get fractions for this from NUTS0 in other year:
greece_codes = ["EL53"]
greece_source = "EL"

excel_data2000 = get_fractions_and_downscale_nuts(cntry_codes=greece_codes,
                                                  parent_nuts_code=greece_source,
                                                  years_to_check=[2018,2021],
                                                  raw_data_curr_year=data_unproc_dict[2000],
                                                  raw_data_other_yr=data_unproc_dict,
                                                  df_to_fill=excel_data2000)

# Portugal NUTS1 also missing, get fractions for these for NUTS0 in other year:
pt_codes = ["PT20", "PT30"]
pt_source = "PT"

excel_data2000 = get_fractions_and_downscale_nuts(cntry_codes=pt_codes,
                                                  parent_nuts_code=pt_source,
                                                  years_to_check=[2006,2018,2021],
                                                  raw_data_curr_year=data_unproc_dict[2000],
                                                  raw_data_other_yr=data_unproc_dict,
                                                  df_to_fill=excel_data2000)

# completely missing
#////////////////////

# malta
mt2006 = np.array([7094031,311187,7405218])
mt2012 =  np.array([7498421,333808,7832229])
diff = mt2012 - mt2006
mt2000 = mt2006 - diff

excel_data2000["OvernightStays"][(excel_data2000["DataAreaCode"] == "MT00") & (excel_data2000["CountryOfResidence"] == "Foreign country")] = mt2000[0]
excel_data2000["OvernightStays"][(excel_data2000["DataAreaCode"] == "MT00") & (excel_data2000["CountryOfResidence"] == "Reporting country")] = mt2000[1]


# estonia
ee2006 = np.array([3020367,1522963,4543330])
ee2012 =  np.array([3823039,1721498,5544537])
diff = ee2012 - ee2006
ee2000 = ee2006 - diff

excel_data2000["OvernightStays"][(excel_data2000["DataAreaCode"] == "EE00") & (excel_data2000["CountryOfResidence"] == "Foreign country")] = ee2000[0]
excel_data2000["OvernightStays"][(excel_data2000["DataAreaCode"] == "EE00") & (excel_data2000["CountryOfResidence"] == "Reporting country")] = ee2000[1]

print("done with 2000 - - - - - - - - - -")
# set all Overnightstays into integers (round if needed)
excel_data2000['OvernightStays'] = np.ceil(excel_data2000['OvernightStays']).astype(int)
# excel_data2000 = excel_data2000.rename(columns={"Unnamed: 0": ""})
excel_data2000.to_csv(os.path.join(current_path, "correctMissingData/overnight_stays_{}.csv".format(2000)),
                      index=False,
                      header=True)


# 2006
# ----
excel_data2006 = data_dict[2006]

# countries on NUTS0 level
# ////////////////////////
raw_data = data_unproc_dict[2006]
list_nuts0_to_correct = ["IE"]
years_to_check = [2012,2018,2021]
types = ['Foreign country', 'Reporting country']

# loop over MS
for ms in list_nuts0_to_correct:
    # grab all NUTS2 per MS
    ms_data = excel_data2006[excel_data2006['DataAreaCode'].str.contains(ms)]

    # split up reporting/ foreign
    for type_rep in types:
        ms_data_type =  ms_data[ms_data["CountryOfResidence"] == type_rep]

        # for each year, get the % of spread of visits
        for yr in years_to_check:
            data = data_dict[yr] # read data
            ms_data_yr = data[data['DataAreaCode'].str.contains(ms)] # get the ms data
            ms_data_yr = ms_data_yr[ms_data_yr["CountryOfResidence"] == type_rep]

            if ms_data_yr["OvernightStays"].sum() == 0:
                ms_data_type["spread_{}".format(yr)] = np.nan
            else:
                ms_data_type["spread_{}".format(yr)] = ms_data_yr["OvernightStays"]/ms_data_yr["OvernightStays"].sum() # calc fractions


        # make the average of this spread
        ms_data_type["avg_spread"] =  ms_data_type[['spread_2012', 'spread_2012', 'spread_2021']].mean(axis=1)

        # apply this average spread to total nuts0 for the missing year
        total_NUTS2_type_report = raw_data[raw_data["nutscode"] == ms]
        if type_rep == "Foreign country":
            type_b = "foreign"
        else:
            type_b = "reporting"
        ms_data_type["OvernightStaysCorr"] = total_NUTS2_type_report[type_b].values[0] * ms_data_type["avg_spread"]
        joinList = ms_data_type["Unnamed: 0"]

        excel_data2006.loc[excel_data2006['Unnamed: 0'].isin(joinList),"OvernightStays"] = ms_data_type.loc[ms_data_type['Unnamed: 0'].isin(joinList), "OvernightStaysCorr"]

        print('a')

# sporadic missing data
# //////////////////////

# Greece NUTS1 also missing, get fractions for this from NUTS0 in other year:
greece_codes = ["EL41","EL53"]
greece_source = "EL"

excel_data2006 = get_fractions_and_downscale_nuts(cntry_codes=greece_codes,
                                                  parent_nuts_code=greece_source,
                                                  years_to_check=[2018,2021],
                                                  raw_data_curr_year=data_unproc_dict[2006],
                                                  raw_data_other_yr=data_unproc_dict,
                                                  df_to_fill=excel_data2006)

# RO 21
# /////
excel_data2006["OvernightStays"][(excel_data2006["DataAreaCode"] == "RO21") & (excel_data2006["CountryOfResidence"] == "Foreign country")] = RO21[2006]["foreign"]
excel_data2006["OvernightStays"][(excel_data2006["DataAreaCode"] == "RO21") & (excel_data2006["CountryOfResidence"] == "Reporting country")] = RO21[2006]["reporting"]

print("done with 2006 - - - - - - - - - -")
excel_data2006['OvernightStays'] = np.ceil(excel_data2006['OvernightStays']).astype(int)
# excel_data2006 = excel_data2006.rename(columns={"Unnamed: 0": ""})
excel_data2006.to_csv(os.path.join(current_path, "correctMissingData/overnight_stays_{}.csv".format(2006)),
                      index=False,
                      header=True)


# 2012
# ----
excel_data2012 = data_dict[2012]

# countries on NUTS0 level
# ////////////////////////
raw_data = data_unproc_dict[2012]
list_nuts0_to_correct = ["IE"]
years_to_check = [2018,2021]
types = ['Foreign country', 'Reporting country']

# loop over MS
for ms in list_nuts0_to_correct:
    # grab all NUTS2 per MS
    ms_data = excel_data2012[excel_data2012['DataAreaCode'].str.contains(ms)]

    # split up reporting/ foreign
    for type_rep in types:
        ms_data_type =  ms_data[ms_data["CountryOfResidence"] == type_rep]

        # for each year, get the % of spread of visits
        for yr in years_to_check:
            data = data_dict[yr] # read data
            ms_data_yr = data[data['DataAreaCode'].str.contains(ms)] # get the ms data
            ms_data_yr = ms_data_yr[ms_data_yr["CountryOfResidence"] == type_rep]

            if ms_data_yr["OvernightStays"].sum() == 0:
                ms_data_type["spread_{}".format(yr)] = np.nan
            else:
                ms_data_type["spread_{}".format(yr)] = ms_data_yr["OvernightStays"]/ms_data_yr["OvernightStays"].sum() # calc fractions


        # make the average of this spread
        ms_data_type["avg_spread"] =  ms_data_type[['spread_2018', 'spread_2021']].mean(axis=1)

        # apply this average spread to total nuts0 for the missing year
        total_NUTS2_type_report = raw_data[raw_data["nutscode"] == ms]
        if type_rep == "Foreign country":
            type_b = "foreign"
        else:
            type_b = "reporting"
        ms_data_type["OvernightStaysCorr"] = total_NUTS2_type_report[type_b].values[0] * ms_data_type["avg_spread"]
        joinList = ms_data_type["Unnamed: 0"]

        excel_data2012.loc[excel_data2012['Unnamed: 0'].isin(joinList),"OvernightStays"] = ms_data_type.loc[ms_data_type['Unnamed: 0'].isin(joinList), "OvernightStaysCorr"]

        print('a')


# fully missing data (not even NUTS0) -> get trend
# ////////////////////////////////////////////////

def fill_gap_trend(data_before,data_after,before_flag,after_flag,cntry_code,data_to_fill):
    """
    before/after_flag == 1: dat coming from processed overnight_stays_xxx.csv
    if 0, it comes from the raw "voorbereidende inputtabellen xxx ninx.xlsx
    so we need a different processing, since tables are formated differently

    :return:
    """
    types = ['Foreign country', 'Reporting country']

    # get all nuts2 of the country that needs to be processed
    if before_flag:
        before_area_code = "DataAreaCode"
    else:
        before_area_code = "nutscode"

    if after_flag:
        after_area_code = "DataAreaCode"
    else:
        after_area_code = "nutscode"

    nuts2_before = data_before[data_before[before_area_code].str.contains(cntry_code)]
    nuts2_after = data_after[data_after[after_area_code].str.contains(cntry_code)]


    for type in types:
        if before_flag:
            nuts2_before_type = nuts2_before[nuts2_before["CountryOfResidence"] == type]
        else:
            if type == "Foreign country":
                type_raw = "foreign"
            else:
                type_raw = "reporting"
            nuts2_before_type["OvernightStays"] = nuts2_before[type_raw]

        if after_flag:
            nuts2_after_type = nuts2_after[nuts2_after["CountryOfResidence"] == type]
        else:
            if type == "Foreign country":
                type_raw = "foreign"
            else:
                type_raw = "reporting"
            nuts2_after_type = nuts2_after
            nuts2_after_type["OvernightStays"] = nuts2_after[type_raw]

        # prep results df
        result = nuts2_before_type[[before_area_code, "Unnamed: 0"]]

        df_prep_change_calc = nuts2_before_type.merge(nuts2_after_type[[after_area_code,"OvernightStays"]],how="left",left_on=before_area_code,right_on=after_area_code)
        result["change"] = list(df_prep_change_calc["OvernightStays_y"] - df_prep_change_calc["OvernightStays_x"])
        result["datatofill"] =  nuts2_before_type["OvernightStays"] + (result["change"]/2) # assuming there was 12 years between change, and we need to interpolate at the halfway point
        joinList = nuts2_before_type['Unnamed: 0']

        # enter the data
        data_to_fill.loc[data_to_fill['Unnamed: 0'].isin(joinList),"OvernightStays"] = result.loc[result['Unnamed: 0'].isin(joinList),"datatofill"]

    return data_to_fill


# greece
fill_gap_trend(data_before=excel_data2006,
               data_after=data_unproc_dict[2018],
               before_flag=1,
               after_flag=0,
               cntry_code="EL",
               data_to_fill=excel_data2012)


# Portugal NUTS1 also missing, get fractions for these for NUTS0 in other year:
pt_codes = ["PT20", "PT30"]
pt_source = "PT"

excel_data2012 = get_fractions_and_downscale_nuts(cntry_codes=pt_codes,
                                                  parent_nuts_code=pt_source,
                                                  years_to_check=[2018,2021],
                                                  raw_data_curr_year=data_unproc_dict[2012],
                                                  raw_data_other_yr=data_unproc_dict,
                                                  df_to_fill=excel_data2012)

# RO 21
# /////
excel_data2012["OvernightStays"][(excel_data2012["DataAreaCode"] == "RO21") & (excel_data2012["CountryOfResidence"] == "Foreign country")] = RO21[2012]["foreign"]
excel_data2012["OvernightStays"][(excel_data2012["DataAreaCode"] == "RO21") & (excel_data2012["CountryOfResidence"] == "Reporting country")] = RO21[2012]["reporting"]


print("done with 2012 - - - - - - - - - -")
excel_data2012['OvernightStays'] = np.ceil(excel_data2012['OvernightStays']).astype(int)
# excel_data2012 = excel_data2012.rename(columns={"Unnamed: 0": ""})
excel_data2012.to_csv(os.path.join(current_path, "correctMissingData/overnight_stays_{}.csv".format(2012)),
                      index=False,
                      header=True)

# 2018
# ----
excel_data2018 = data_dict[2018]

#France - only totals => get spread foreign/reporting from other years
list_fr = ["FRC1","FRD2","FRE2","FRF2","FRI2"]
corrected_Data = pd.read_excel("fractions_foreign_reporting_france.xlsx")
types = ['Foreign country', 'Reporting country']

for nuts in list_fr:
    for type in types:
        data2fill = corrected_Data[nuts][corrected_Data["foreign/total fracs"] == type].values[0]
        excel_data2018["OvernightStays"][(excel_data2018["DataAreaCode"] == nuts) & (excel_data2018["CountryOfResidence"] == type)] = data2fill


# slovenia
fill_gap_trend(data_before=excel_data2012,
               data_after=data_unproc_dict[2021],
               before_flag=1,
               after_flag=0,
               cntry_code="SI",
               data_to_fill=excel_data2018)

# RO 21
# /////
excel_data2018["OvernightStays"][(excel_data2018["DataAreaCode"] == "RO21") & (excel_data2018["CountryOfResidence"] == "Foreign country")] = RO21[2018]["foreign"]
excel_data2018["OvernightStays"][(excel_data2018["DataAreaCode"] == "RO21") & (excel_data2018["CountryOfResidence"] == "Reporting country")] = RO21[2018]["reporting"]


print("done with 2018 - - - - - - - - - -")
excel_data2018['OvernightStays'] = np.ceil(excel_data2018['OvernightStays']).astype(int)
# excel_data2018 = excel_data2018.rename(columns={"Unnamed: 0": ""})
excel_data2018.to_csv(os.path.join(current_path, "correctMissingData/overnight_stays_{}.csv".format(2018)),
                      index=False,
                      header=True)

# 2021
# ----
excel_data2021 = data_dict[2021]

# missing data germany
ger_codes = ["DEG0"]
ger_source = "DE"
years_to_check = [2006,2012,2018]
excel_data2021 = get_fractions_and_downscale_nuts(cntry_codes=ger_codes,
                                                  parent_nuts_code=ger_source,
                                                  years_to_check=years_to_check,
                                                  raw_data_curr_year=data_unproc_dict[2021],
                                                  raw_data_other_yr=data_unproc_dict,
                                                  df_to_fill=excel_data2021)

# RO 21
# /////
excel_data2021["OvernightStays"][(excel_data2021["DataAreaCode"] == "RO21") & (excel_data2021["CountryOfResidence"] == "Foreign country")] = RO21[2021]["foreign"]
excel_data2021["OvernightStays"][(excel_data2021["DataAreaCode"] == "RO21") & (excel_data2021["CountryOfResidence"] == "Reporting country")] = RO21[2021]["reporting"]


print("done with 2021 - - - - - - - - - -")
excel_data2021['OvernightStays'] = np.ceil(excel_data2021['OvernightStays']).astype(int)
# excel_data2021 = excel_data2021.rename(columns={"Unnamed: 0": ""})
excel_data2021.to_csv(os.path.join(current_path, "correctMissingData/overnight_stays_{}.csv".format(2021)),
                      index=False,
                      header=True)

print("all done")

