import pandas as pd
import geopandas as gpd
import numpy as np
import os

current_path = os.getcwd()
def nearly_identical(geoms, p):
    nearly = (geoms.intersection(p).area / p.area) > 0.75
    # return index values where nearly is True
    return pd.Series(nearly.index[nearly])

# nuts data shapes
nuts2021 = r"C:\repos\inca\src\inca\data\NUTS_RG_03M_2021_3035_LEVL012_EU2020.gpkg"
nuts2013 = r"C:\repos\inca_geopandas\NUTS_RG_03M_2013_3035.shp\NUTS_RG_03M_2013_3035.shp"
nuts2016 = r"C:\repos\inca_geopandas\NUTS_RG_03M_2016_3035.shp\NUTS_RG_03M_2016_3035.shp"

nutsshapes = {
    2013: nuts2013,
    2016: nuts2016,
    2021: nuts2021
}

shapefiles = dict()
for year, path in nutsshapes.items():
    shapefiles[year] = gpd.read_file(path)


years2process = [2000,2006,2012,2018,2021]
file2process = "data_nuts_overnight/Voorbereiding inputtabellen {} nin2.xlsx"

for year in years2process:
    if year != 2021:
        file2load = file2process.format(year)
    else:
        file2load = "data_nuts_overnight/Voorbereiding inputtabellen 2021 nin2d.xlsx"
    data = pd.read_excel(file2load,
                         sheet_name="Basistabel",
                         skiprows=4)
    if year in [2000,2006,2012]:
        data = data.iloc[:, :-2]

        # rename cols
        data.columns=["nutscode",
                    "nutsname",
                    "foreign",
                    "reporting",
                    "total"]

    elif year == 2018:
        data = data.iloc[:, [0,1,2,4,6]]

        # rename cols
        data.columns=["nutscode",
                    "nutsname",
                    "foreign",
                    "total",
                    "reporting"]
    elif year == 2021:
        data = data.iloc[:, [1,2,3,7,11]]

        # rename cols
        data.columns = ["nutscode",
                        "nutsname",
                        "foreign",
                        "reporting",
                        "total"]



    data_final = pd.read_excel(file2load,
                                sheet_name="TblOvernightStays")


    # create a new colum we will use later to store the names of regions we corrected (to see if the names at least are the same)
    data["check_nutsname"] = ""

    # look for nuts xxxx str in column B
    filtered_rows = data[data["nutsname"].str.contains("NUTS ", case=False)]
    filtered_nutsname = filtered_rows["nutsname"].str.split("NUTS ").str[-1]
    filtered_nutsname2 = filtered_nutsname.str.split(")").str[0]
    filtered_rows["nutsyear"] = filtered_nutsname2.astype(int)

    filtered_rows = filtered_rows[filtered_rows['nutsyear'] != 2010]
    # match that record to the given nuts shape
    for indx,row in filtered_rows.iterrows():
        print("--------------- ROW {} ------------------".format(indx))
        if row["total"] == ":":
            print("skip row, empty data. Year: {} -- nutscode: {}".format(year,row["nutscode"]))
        else:
            nutscode_to_check = row["nutscode"]
            year_to_check = row["nutsyear"]
            shapefile_to_check = shapefiles[year_to_check]
            record_shape = shapefile_to_check[shapefile_to_check["NUTS_ID"] == nutscode_to_check]
            if record_shape["LEVL_CODE"].values[0] == 2:
                # find fuzzy spatial match between that record and 2021 nuts shape
                matches = shapefiles[2021].geometry.apply(lambda x: nearly_identical(record_shape, x))

                # if there was a match, take note of this nutsID2021
                matches2 = matches.unstack().reset_index(0, drop=True).dropna()
                num_rows = len(matches2)
                # Check if the DataFrame is empty
                if num_rows == 0:
                    print("skip row, no matches with 2021. Year: {} -- nutscode: {}".format(year, row["nutscode"]))
                else:
                    print(f"Number of rows: {num_rows}")
                    if num_rows > 1:
                        print("hol'up - - multiple matches Year: {} -- nutscode: {}".format(year, row["nutscode"]))
                        matches2list = list(matches2.index)
                        nutslevels = []
                        nutscodes = []
                        for indices_2021 in matches2list:
                            nutscheck = shapefiles[2021]["LEVL_CODE"][indices_2021]
                            nutslevels.append(nutscheck)
                            nutscodes.append(shapefiles[2021]["NUTS_ID"][indices_2021])


                        num2  = nutslevels.count(2)
                        # number of lvl 2 == 1 ? continue
                        if num2 == 1:
                            indices = [index for index, value in enumerate(nutslevels) if value == 2][0]
                            nuts2021IDind = matches2list[indices]
                            nuts2021ID = shapefiles[2021]["NUTS_ID"][nuts2021IDind]
                            shared_flag = False
                            print("multiple matches Year: {} -- nutscode: {}, due to nuts 1 or 3 overlap, selected nuts2 -- nutscode {} nutslevel {}".format(year, row["nutscode"],nutscodes,nutslevels))

                        # number of lvl 2 > 1? devide visits over amount of shapes with lvl 2
                        else:
                            indices = [index for index, value in enumerate(nutslevels) if value == 2]
                            nuts2021IDind = [matches2list[index] for index in indices]
                            nuts2021ID = shapefiles[2021]["NUTS_ID"][nuts2021IDind] # becomes a list
                            shared_pop = row # contains population data row[foreign,reporting,total]
                            shared_flag = True
                            print("multiple matches Year: {} -- nutscode: {}, SPLIT over {} 2021 nuts shapes: {}".format(year, row["nutscode"],num2,nuts2021ID))

                        matches2["nutslevels"] = nutslevels
                        print(matches2)
                    else:
                        nuts2021IDind = matches2.index.values[0]
                        nuts2021ID = shapefiles[2021]["NUTS_ID"][nuts2021IDind]
                        shared_flag = False

                    # if data was missing for that ID, overwrite the value
                    if shared_flag:
                        if  row["foreign"] == ":":
                            row["foreign"] = 0
                        if row["reporting"] == ":":
                            row["reporting"] = 0
                        if row["total"] == ":":
                            row["total"] = 0
                        foreign_split = row["foreign"]/num2
                        reporting_split = row["reporting"] / num2
                        total_split = row["total"] / num2
                        name_of_new_data = row["nutsname"]
                        for nuts in nuts2021ID:
                            found_potential_issue = data[data["nutscode"] == nuts]
                            if found_potential_issue["total"].values[0] == ":":
                                data["foreign"][data["nutscode"] == nuts] = foreign_split
                                data["reporting"][data["nutscode"] == nuts] = reporting_split
                                data["total"][data["nutscode"] == nuts] = total_split
                                data["check_nutsname"][data["nutscode"] == nuts] = name_of_new_data
                            else:
                                print(" X X X -- skip row, there is already data for 2021 nuts area. Year: {} -- nutscode: {}".format(
                                        year, row["nutscode"]))

                    else:
                        found_potential_issue = data[data["nutscode"] == nuts2021ID]
                        if found_potential_issue["total"].values[0] == ":":
                            newForeign_data = row["foreign"]
                            newReporting_data = row["reporting"]
                            newTotal_data = row["total"]
                            name_of_new_data = row["nutsname"]

                            # execute data insert
                            data["foreign"][data["nutscode"] == nuts2021ID] = newForeign_data
                            data["reporting"][data["nutscode"] == nuts2021ID] = newReporting_data
                            data["total"][data["nutscode"] == nuts2021ID] = newTotal_data
                            data["check_nutsname"][data["nutscode"] == nuts2021ID] = name_of_new_data
                            print(" Inserted row. Year: {} -- nutscode: {}".format(year, row["nutscode"]))

                        else:
                            print(" X X X -- skip row, there is already data for 2021 nuts area. Year: {} -- nutscode: {}".format(year, row["nutscode"]))


            else:
                print("skip row, not NUTS2 area. Year: {} -- nutscode: {}".format(year, row["nutscode"]))

    # write to disk this corrected data sheet
    print("WRITING {}".format(year))
    data.to_csv(os.path.join(current_path,"temp_conversion_by_scripts/nuts_correction_{}.csv".format(year)))

    # write it back to a clean overnights stay csv (TblOvernightStays)
    print("WRITING FINAL {}".format(year))
    if year != 2021:
        for rowind, row in data_final.iterrows():
            if row["OvernightStays"] == ":":
                if row["CountryOfResidence"] == "Foreign country":
                    data_final["OvernightStays"][rowind] = data["foreign"][data["nutscode"] == row["DataAreaCode"]].values[0]
                elif row["CountryOfResidence"] == "Reporting country":
                    data_final["OvernightStays"][rowind] = data["reporting"][data["nutscode"] == row["DataAreaCode"]].values[0]
    elif year == 2021:
        # remove everything but cities
        data_final = data_final[data_final["TypeOfArea"] == "Cities"]
        for rowind, row in data_final.iterrows():
            if row["CountryOfResidence"] == "Foreign country":
                data_final["OvernightStays"][rowind] = \
                data["foreign"][data["nutscode"] == row["DataAreaCode"]].values[0]
            elif row["CountryOfResidence"] == "Reporting country":
                data_final["OvernightStays"][rowind] = \
                data["reporting"][data["nutscode"] == row["DataAreaCode"]].values[0]

    # remove trailing columns
    data_final = data_final.iloc[:,:6]

    # remove blank rows
    data_final = data_final.dropna(how='all')
    data_final["Year"] = year
    data_final['OvernightStays'] = data_final['OvernightStays'].fillna(0)
    data_final['OvernightStays'] = data_final['OvernightStays'].replace(":", 0)

    # remove float visit data, round up
    data_final['OvernightStays'] = data_final['OvernightStays'].apply(np.ceil)
    data_final = data_final.rename(columns = {"Unnamed: 0" : ""})
    data_final['OvernightStays'] = data_final['OvernightStays'].astype(int)

    data_final.to_csv(os.path.join(current_path,"final_csvs_overnightstays/overnight_stays_{}.csv".format(year)),
                      index=False,
                      header=True)

    print("done year: {}".format(year))


print("done")




