# Copyright (c) 2022 European Union.
#
# The tool was developed with the contribution of the Joint Research Centre of the European Commission.
#
# This program is free software: you can redistribute it and/or modify it under the terms of the European Union Public
# Licence, either version 1.2 of the License, or (at your option) any later version.
# You may not use this work except in compliance with the Licence.
#
# You may obtain a copy of the Licence at: https://joinup.ec.europa.eu/collection/eupl/eupl-guidelines-faq-infographics
#
# Unless required by applicable law or agreed to in writing, software distributed under the Licence is distributed on
# an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the Licence for the specific language governing permissions and limitations under the Licence.

"""Remove datapoints where the ratio 'exports in EUR / exports in m3' is regarded as an outlier by JRC."""

import argparse

import pandas as pd

import inca.common.estat as estat
from inca.woodprovision import ROUNDWOOD, TOTAL, EXPORTS, THS_EUR, THS_M3


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('for_basic', help='Original TSV file from Eurostat.', metavar='TSV')
    parser.add_argument('output', help='Filename for resulting adjusted file.')

    return parser.parse_args()


def main():
    args = parse_args()

    for_basic = estat.read_tsv(args.for_basic)

    # Remove DK, 2014 and EL, 2006
    for_basic.loc[(ROUNDWOOD, TOTAL, EXPORTS, THS_EUR, 'DK'), 2014] = pd.NA
    for_basic.loc[(ROUNDWOOD, TOTAL, EXPORTS, THS_M3, 'DK'), 2014] = pd.NA
    for_basic.loc[(ROUNDWOOD, TOTAL, EXPORTS, THS_EUR, 'EL'), 2006] = pd.NA
    for_basic.loc[(ROUNDWOOD, TOTAL, EXPORTS, THS_M3, 'EL'), 2006] = pd.NA

    estat.write_tsv(for_basic, args.output)


if __name__ == '__main__':
    main()
